/***
	Recycle.prg
	
	Use in place of standard locks.prg in order to recycle
	records marked for deletion.
	
	Copyright (c) 1990, Nantucket Corp.  All rights reserved.
	
	Author:    Orlando Chacon
	Note:      Compile with /N /W switches.
	
***/

#define RECYCLE_KEY "#" // Key value for recycled records

// To identify data TYPE() of  fields when purging record contents
#define TYPE_CHARACTER "C"  
#define TYPE_DATE      "D"        	
#define TYPE_LOGICAL   "L"	// |
#define TYPE_NUMERIC   "N"	// |
#define TYPE_MEMO      "M"	// +

FUNCTION AddRec( nWaitSeconds )
	/***
	AddRec( <nWaitSeconds> ) _> lSuccess
	Attempts to recycle deleted records before APPENDing BLANK
*/

	LOCAL lForever
	LOCAL lOldDeleted := SET( _SET_DELETED, .F.)
	LOCAL lSuccess    := .F.
	LOCAL lSoft       := SET( _SET_SOFTSEEK, .F. )
	LOCAL nFieldCount := 0
	LOCAL nCurrField  := 0
	BEGIN SEQUENCE

		SEEK RECYCLE_KEY
		IF FOUND() .AND. DELETED()
			IF RecLock( nWaitSeconds )
				RECALL
				lSuccess := .T.

				// Remove Recyle key value from character fields
				nFieldCount := FCOUNT()
				FOR nCurrField := 1 to nFieldCount
					IF TYPE( FIELD( nCurrField ) ) == TYPE_CHARACTER
						FIELDPUT( nCurrField, SPACE(1) )
             					ENDIF
            				NEXT
            				BREAK

         			ELSE 
            				BREAK
         			ENDIF

      		ELSE
         			APPEND BLANK
         			IF !NETERR()
            				IF RecLock( nWaitSeconds )
               					lSuccess := .T.
               					BREAK
            				ENDIF
			ENDIF

			lForever := (nWaitSeconds = 0)
         			DO WHILE (lForever .OR. nWaitSeconds > 0)

            				APPEND BLANK
            				IF !NETERR()
               					IF RecLock( nWaitSeconds )
                  						lSuccess := .T.
                  						BREAK
               					ENDIF
            				ENDIF

            				INKEY(.5)         // Wait 1/2 second
				nWaitSeconds  = nWaitSeconds  - .5

         			ENDDO

      		ENDIF // Found record?

	END SEQUENCE

	// Restore environment

	SET( _SET_DELETED, lOldDeleted )
	SET( _SET_SOFTSEEK, lSoft )

	RETURN ( lSuccess )

FUNCTION RecLock( nSeconds )
	/***
	Standard RecLock function from locks.prg
	RecLock( <nWaitSeconds> ) _> lSuccess
	Attempt to RLOCK() with optional retry
	***/
	LOCAL lForever
	IF RLOCK()
		RETURN (.T.)        // Locked
	ENDIF
	lForever = (nSeconds = 0)
	DO WHILE (lForever .OR. nSeconds > 0)
		IF RLOCK()
			RETURN (.T.)     // Locked
		ENDIF
		INKEY(.5)           // Wait 1/2 second
      nSeconds = nSeconds - .5
	ENDDO
	RETURN (.F.)           // Not locked

FUNCTION DeleteRec( nWaitSeconds )
	/***
	DeleteRec( <nWaitSeconds> ) _> lSuccess
	Purge record, mark for deletion, and flag for recycling
	***/

	LOCAL nNumberofFields := FCOUNT()
	LOCAL lSuccess        := .F.
	LOCAL nCurField       := 0

	nWaitSeconds := IF( nWaitSeconds=NIL, 0, nWaitSeconds )
	// Lock this record
	IF RecLock( nWaitSeconds )
		// Purge each field based on data type
		FOR nCurField := 1 to nNumberofFields
			DO CASE
			// Assumes all keys are of type character
			CASE TYPE( FIELD(nCurField) ) = TYPE_CHARACTER
				FIELDPUT( nCurField, RECYCLE_KEY )
			CASE TYPE( FIELD(nCurField) ) = TYPE_DATE
           				FIELDPUT( nCurField, CTOD(" ") )
         			CASE TYPE( FIELD(nCurField) ) = TYPE_LOGICAL
           				FIELDPUT( nCurField, .F. )
         			CASE TYPE( FIELD(nCurField) ) = TYPE_NUMERIC
           				FIELDPUT( nCurField, 0 )
         			CASE TYPE( FIELD(nCurField) ) = TYPE_MEMO
           				IF !EMPTY( FIELDGET( nCurField ) )
             					FIELDPUT( nCurField, SPACE(1) )
           				ENDIF
       			ENDCASE
     		NEXT
     		DELETE
     		lSuccess := .T.
     		UNLOCK
   	ENDIF

	RETURN( lSuccess )

FUNCTION PostToDeletionLog()
	/***
  Posts a copy of the record to be deleted to a deletion
  log.  See Function MakeDelLog() for field definitions.
	***/
  	LOCAL cSourceAlias := ALIAS() // Identifies Source Table
	// Identifies User performing deletion
  LOCAL cUser := GETENV( "USR" )  
	// | DOS environmental variable name
	LOCAL dDelDate := DATE() 
	LOCAL cDelTime := TIME() // + may be different
	LOCAL nLastSelect  := SELECT()
	LOCAL lSuccess     := .F.

	SELECT DeletLog

	APPEND BLANK

	IF !NETERR()
		DeletLog->TABLE     := cSourceAlias
		DeletLog->DEL_BY    := cUser       
		DeletLog->DEL_DATE  := dDelDate    
		DeletLog->DEL_TIME  := cDelTime    
		DeletLog->OLDRECORD := RecToMemo( cSourceAlias )
		lSuccess := .T.
		UNLOCK
	ENDIF

	SELECT ( nLastSelect )

	RETURN( lSuccess )

FUNCTION RecToMemo( cSourceAlias )
	/***
  Converts record to a character string to be stored in a
  MEMO field.  Note FieldSep (~) used to delimit fields.  See
  function MemoToRec for extracting stored Memos back
  to original format.
	***/
	LOCAL nFieldCount     := 0
	LOCAL cRecString      := ""
	LOCAL cFieldSep       := "~"
	LOCAL nCurrField      := 0
	SELECT ( cSourceAlias )
	nFieldCount := FCOUNT()

	FOR nCurrField := 1 to nFieldCount
		DO CASE

		CASE TYPE( FIELD(nCurrField) ) = TYPE_CHARACTER
    // Field is type character, perform string transform in
    // current field replacing any occurences of the
    // cFieldSep to spaces
		cRecString := cRecString + ;
			STRTRAN( ALLTRIM( (cSourceAlias)-> ;
			( FIELDGET( nCurrField ) ) ), ;
			cFieldSep, SPACE(1) ) + cFieldSep

		CASE TYPE( FIELD(nCurrField) ) = TYPE_DATE
			cRecString := cRecString + ;
			DTOC( (cSourceAlias)->( FIELDGET( nCurrField ) ) ) + ;
			cFieldSep

		CASE TYPE( FIELD(nCurrField) ) = TYPE_LOGICAL
			cRecString := cRecString + ;
				IF( (cSourceAlias)->( FIELDGET( nCurrField ) ), ;
				"T","F" ) + cFieldSep

		CASE TYPE( FIELD(nCurrField) ) = TYPE_NUMERIC
			cRecString := cRecString + ;
			ALLTRIM( STR( (cSourceAlias)-> ;
			( FIELDGET( nCurrField ) ) ) ) + cFieldSep

		CASE TYPE( FIELD(nCurrField) ) = TYPE_MEMO
      // Field is type memo, perform string transform in
      // current field replacing any occurences of the
      //  cFieldSep to spaces
			cRecString := cRecString + ;
			STRTRAN( ALLTRIM( (cSourceAlias)-> ;
			( FIELDGET( nCurrField ) ) ),;
			cFieldSep, SPACE(1) ) + cFieldSep

		OTHERWISE
			cRecString := cRecString + ;
				"Error: Unknown type!" + cFieldSep
		ENDCASE
 	NEXT
	RETURN( cRecString )


FUNCTION MemoToRec( cTargetAlias, cRecString )
	/***
	Decompresses record information created by RecToMemo().
	Returns array aFieldValues which can be used to restore 
	record information back into original format as shown:
	**** Sample Record Restoration 
	
	aFieldVals := ;
		MemoToRec( "TargetFile", DeletLog->OldRecord )
	nFCount := TargetFile->( FCOUNT() )
	SELECT TargetFile
	IF AddRec(5)
		FOR x := 1 to nFCount
		FIELDPUT( x, aFieldVals[x] )
		ENDIF
	ENDIF
	***/
	LOCAL nFieldCount     := 0
	LOCAL cFieldSep       := "~"
	LOCAL cBuffer         := cRecString
	LOCAL nCurrField      := 0
	LOCAL nChop           := 0
	LOCAL aRecord         := {}
	LOCAL cCurrField      := SPACE(10)

	SELECT ( cTargetAlias )
	nFieldCount := FCOUNT()

	FOR nCurrField := 1 to nFieldCount
		nChop      := AT( cFieldSep, cRecString )
		cCurrField := SUBSTR( cBuffer, 1, nChop - 1 )
		cBuffer    := SUBSTR( cBuffer, nChop + 1 )

		DO CASE
		CASE TYPE( FIELD(nCurrField) ) = TYPE_CHARACTER
			AADD( aRecord, cCurrField )

		CASE TYPE( FIELD(nCurrField) ) = TYPE_DATE
			AADD( aRecord, CTOD( cCurrField ) )

		CASE TYPE( FIELD(nCurrField) ) = TYPE_LOGICAL
			AADD( aRecord, IF( cCurrField == "T", .T., .F. ) )

		CASE TYPE( FIELD(nCurrField) ) = TYPE_NUMERIC
			AADD( aRecord, VAL( cCurrField ) )

		CASE TYPE( FIELD(nCurrField) ) = TYPE_MEMO
			AADD( aRecord, cCurrfield )
		OTHERWISE
			AADD( aRecord, "Error: Unknown type!")
		 ENDCASE
	 NEXT
	RETURN( aRecord )

FUNCTION MakeDelLog()
	/***
	Creates deletion log table
	***/ 
  	LOCAL aDeleteLog :=	{ ;
                        { "TABLE"     , "C", 12, 0 }, ;
                        { "DEL_BY"    , "C", 15, 0 }, ;
                        { "DEL_DATE"  , "D",  8, 0 }, ;
                        { "DEL_TIME"  , "C",  8, 0 }, ;
                        { "OLDRECORD" , "M", 10, 0 }  }
   
  DBCREATE( "DeletLog", aDeleteLog )


	RETURN( NIL )
